part of 'login_bloc.dart';

abstract class LoginState extends Equatable {
  const LoginState();

  @override
  List<Object> get props => [];
}

class LoginInitial extends LoginState {
  const LoginInitial();
}

class LoginLoading extends LoginState {
  const LoginLoading();
}

class LoginReady extends LoginState {
  const LoginReady();
}

class LoginSuccess extends LoginState {
  const LoginSuccess();
}

class LoginSkipped extends LoginState {
  const LoginSkipped();
}

class LoginError extends LoginState {
  final String message;
  const LoginError({required this.message});

  @override
  List<Object> get props => [message];
}

class DataPolicyError extends LoginState {
  const DataPolicyError();
}
